﻿Type.registerNamespace('Jeffz.Net');

Jeffz.Net.ServiceMethodRequest = function() {
	Jeffz.Net.ServiceMethodRequest.initializeBase(this);
	
	var _url = null;
	var _appUrl = null;
	var _methodName = null;
	var _parameters = null;
	var _response = null;
	var _userContext = null;
	var _result = null;
	var _request = null;
	var _timeoutInterval = 0;
	var _priority = Sys.Net.WebRequestPriority.Normal;
	
	var _onCompleteHander = null;
	var _onErrorHander = null;

	this.get_url = function() {
		return _url;
	}
	this.set_url = function(value) {
		_url = value;
	}

	this.get_appUrl = function() {
		return _appUrl;
	}
	this.set_appUrl = function(value) {
		_appUrl = value;
	}
	
	this.get_methodName = function() {
		return _methodName;
	}
	this.set_methodName = function(value) {
		_methodName = value;
	}

	this.get_parameters = function() {
		if (_parameters == null) {
			_parameters = Sys.Component.createCollection(this);
		}
		return _parameters;
	}

	this.get_response = function() {
		return _response;
	}

	this.get_result = function() {
		return _result;
	}

	this.get_timeoutInterval = function() {
		return _timeoutInterval;
	}
	
	this.set_timeoutInterval = function(value) {
		_timeoutInterval = value;
	}

	this.get_priority = function() {
		return _priority;
	}

	this.set_priority = function(value) {
		_priority = value;
	}
	

	this.completed = this.createEvent();
	this.timeout = this.createEvent();
	this.error = this.createEvent();
	this.aborted = this.createEvent();

	this.invoke = function(userContext) {
		if (_request != null) {
			return false;
		}
		
		var params = new Object();
		for (var i = 0; i < _parameters.length; i++)
		{
			params[_parameters[i].get_name()] = _parameters[i].get_value()
		}

		if (_onCompleteHander == null)
		{
			_onCompleteHander = Function.createDelegate(this, onMethodComplete);
		}
		
		if (_onErrorHander == null)
		{
			_onErrorHander = Function.createDelegate(this, onMethodError);
		}
			
		var serviceMethod = new Sys.Net.ServiceMethod(_url, _methodName, _appUrl);
		_request = serviceMethod.invoke(params, _onCompleteHander, onMethodTimeout, 
			_onCompleteHander, onMethodAborted, this ,
			_timeoutInterval, _priority);
			
		
		function onMethodComplete(result, response, target) {
			_request = null;
			_response = response;
			_result = result;
			_userContext = userContext;
			this.raisePropertyChanged("result");
			target.completed.invoke(target, Sys.EventArgs.Empty);
		}

		function onMethodError(result, response, target) {
			_request = null;
			_response = response;
			_result = result;
			_userContext = userContext;
			this.raisePropertyChanged("result");
			target.error.invoke(target, Sys.EventArgs.Empty);
		}

		function onMethodTimeout(request, target ) {
			_request = null;
			_userContext = userContext;
			target.timeout.invoke(request, Sys.EventArgs.Empty);
		}
		
		function onMethodAborted(request, target ) {
			_request = null;
			_userContext = userContext;
			target.aborted.invoke(request, Sys.EventArgs.Empty);
		}
		
		return true;
	}
	Jeffz.Net.ServiceMethodRequest.registerBaseMethod(this, 'invoke');

	this.abort = function() {
		if (_request) {
			_request.abort();
		}
	}
	

	this.getDescriptor = function() {
		var td = Jeffz.Net.ServiceMethodRequest.callBaseMethod(this, 'getDescriptor');
		
		td.addProperty('url', String);
		td.addProperty('appUrl', String);
		td.addProperty('methodName', String);
		td.addProperty('parameters', Array, true);
		td.addProperty('response', Sys.Net.WebRequestExecutor, true);
		td.addProperty('result', Object, true);
		td.addProperty('timeoutInterval', Number);
		td.addProperty('priority', Sys.Net.WebRequestPriority);

		td.addMethod('invoke');
		td.addMethod('abort');

		td.addEvent('completed', true);
		td.addEvent('timeout', true);
		td.addEvent('error', true);
		td.addEvent('aborted', true);

		return td;
	}
	Jeffz.Net.ServiceMethodRequest.registerBaseMethod(this, 'getDescriptor');
	
	
	this.dispose = function() {
        _parameters.dispose();
        _parameters = null;
        
        _onCompleteHander = null;
		_onErrorHander = null;
		
		Jeffz.Net.ServiceMethodRequest.callBaseMethod(this, 'dispose');
    }
    Jeffz.Net.ServiceMethodRequest.registerBaseMethod(this, 'dispose');
}
Jeffz.Net.ServiceMethodRequest.registerClass('Jeffz.Net.ServiceMethodRequest', Sys.Component);
Sys.TypeDescriptor.addType('jeffz', 'serviceMethod', Jeffz.Net.ServiceMethodRequest);


Jeffz.Net.Parameter = function()
{
	Jeffz.Net.Parameter.initializeBase(this);
	
	var _name = null;
	var _value = null;
	
	this.get_name = function()
	{
		return _name;
	}
	
	this.set_name = function(value)
	{
		_name = value;
	}
	
	this.get_value = function()
	{
		return _value;
	}
	
	this.set_value = function(value)
	{
		_value = value;
	}
	
	this.setOwner = function(owner) {
        this._owner = owner;
    }
	
	this.getDescriptor = function() {
		var td = Jeffz.Net.Parameter.callBaseMethod(this, 'getDescriptor');
		
		td.addProperty('name', String);
		td.addProperty('value', String);
		
		return td;
	}
	Jeffz.Net.ServiceMethodRequest.registerBaseMethod(this, 'getDescriptor');
}
Jeffz.Net.Parameter.registerSealedClass('Jeffz.Net.Parameter', Sys.Component);
Sys.TypeDescriptor.addType('jeffz', 'param', Jeffz.Net.Parameter);